# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test template tags
"""
from datetime import datetime
from django.core.exceptions import ImproperlyConfigured
from django.template import Template, Context
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.translation import activate

from ..models import Category, Page
from .mixins import TDCMSTestMixin


class TDCMSTemplateTagTests(TDCMSTestMixin, TestCase):
    """
    Tests for the td_cms_tags template tags library
    """
    def setUp(self):
        activate('en')
        self._set_category_tree()

    def test_get_parent_categories(self):
        """
        Retrieve visible categories to display in a (mptt) rendered menu
        """
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_parent_categories as categories %}"
            "{% for category in categories %}"
            "{% if category.is_child_node %}"
            "> {{ category }} "
            "{% else %}"
            "{{ category }} "
            "{% endif %}"
            "{% endfor %}"
        )

        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = "".join(
            "Django books > Testing in Django > Jazz with Django "
            "Django tee > Keep calm > Poney ride "
            "Django stickers > Poney mascotte > Django logo "
        )
        self.assertEqual(out, expected)

        # Test with a hidden category in menu
        Category.objects.filter(name='Keep calm').update(visible_in_menu=False)
        out = Template(template).render(Context())
        expected = "".join(
            "Django books > Testing in Django > Jazz with Django "
            "Django tee > Poney ride "
            "Django stickers > Poney mascotte > Django logo "
        )
        self.assertEqual(out, expected)

    def test_get_highlighted_pages(self):
        """
        Get a queryset of all highlighted pages in templates
        """
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_highlighted_pages as pages %}"
            "{% for page in pages %}"
            "{{ page }} - "
            "{% endfor %}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = "".join(
            "Django book (vol. 8) - "
            "Django book (vol. 16) - "
            "Django book (vol. 24) - "
            "Django book (vol. 32) - "
        )
        self.assertEqual(out, expected)

    def test_get_category(self):
        """
        Get a category object by its slug in a template. We test two different
        languages (fr, en).
        """
        # Add translations for our category of interest
        category = Category.objects.get(name='Jazz with Django')
        category.name_fr = u'Jaser avec Django'
        category.save()

        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_category 'jazz-with-django' as category %}"
            "{{ category }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = category.name
        self.assertEqual(out, expected)

        # Switch to french
        activate('fr')

        # Test the french slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_category 'jaser-avec-django' as category %}"
            "{{ category }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = category.name_fr
        self.assertEqual(out, expected)

    def test_get_category_with_unknown_slug(self):
        """
        Try to get a category with a wrong slug
        """
        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_category 'swing-with-django' as category %}"
            "{{ category }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = u"None"
        self.assertEqual(out, expected)

    def test_get_page(self):
        """
        Get a page object by its slug in a template. We test two different
        languages (fr, en).
        """
        # Add translations for our category of interest
        page = Page.objects.get(title='Django book (vol. 10)')
        page.title_fr = u'Le livre Django (vol. 10)'
        page.save()

        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_page 'django-book-vol-10' as page %}"
            "{{ page }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = page.title
        self.assertEqual(out, expected)

        # Switch to french
        activate('fr')

        # Test the french slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_page 'le-livre-django-vol-10' as page %}"
            "{{ page }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = page.title_fr
        self.assertEqual(out, expected)

    def test_get_page_with_unknown_slug(self):
        """
        Try to get a page with a wrong slug
        """
        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_page 'php-book-vol-1' as page %}"
            "{{ page }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = u"None"
        self.assertEqual(out, expected)

    def test_timestamp_filter(self):
        """
        Get a timestamp from a datetime.datetime object
        """
        date = datetime(year=1980, month=10, day=14, hour=18, minute=30)
        template = "".join(
            "{% load td_cms_tags %}"
            "{{ date|timestamp }}"
        )
        # Test the datetime to timestamp conversion
        out = Template(template).render(Context({'date': date}))
        expected = u"340392600"
        self.assertEqual(out, expected)

    def test_timestamp_filter_without_datetime_instance(self):
        """
        Try to get a timestamp from a non datetime.datetime object, should be
        returned identical.
        """
        date = u"14/10/1980"
        template = "".join(
            "{% load td_cms_tags %}"
            "{{ date|timestamp }}"
        )
        # Test the datetime to timestamp conversion
        out = Template(template).render(Context({'date': date}))
        expected = date
        self.assertEqual(out, expected)

    def test_show_disqus_thread_while_not_configured(self):
        """
        Test the disqus thread inclusion while not configured
        """
        page = Page.objects.get(title='Django book (vol. 10)')
        template = "".join(
            "{% load td_cms_tags %}"
            "{% show_disqus_thread page %}"
        )
        # Raises an error while not configured
        with self.assertRaises(ImproperlyConfigured):
            Template(template).render(Context({'page': page}))

    @override_settings(TD_CMS_DISQUS_SHORTNAME='tailordev')
    def test_aaa_show_disqus_thread(self):
        """
        Test the disqus thread inclusion while not configured
        """
        page = Page.objects.get(title='Django book (vol. 10)')
        template = "".join(
            "{% load td_cms_tags %}"
            "{% show_disqus_thread page %}"
        )
        out = Template(template).render(Context({'page': page}))
        expected = "var disqus_shortname = 'tailordev';"
        self.assertIn(expected, out)
        expected = "var disqus_identifier = '%d-%s';" % (page.id, page.slug)
        self.assertIn(expected, out)
        expected = "var disqus_title = '%s';" % page.title
        self.assertIn(expected, out)



# -*- coding: utf-8 -*-
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.utils.translation import get_language, ugettext as _
from django.views.generic import DetailView

from .models import Category, Page


class TranslatedSlugMixin(object):
    """
    Returns the object the view is displaying, given a translated slug.
    """

    def get_object(self, queryset=None):
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        # Current language translated slug name
        language = get_language().split('-')[0].lower()
        slug_field = 'slug_%s' % language

        # The translated slug value matching our url pattern
        slug = self.kwargs.get(slug_field, None)

        # Our object
        queryset = queryset.filter(**{slug_field: slug})

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except ObjectDoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


class CategoryDetailView(TranslatedSlugMixin, DetailView):
    model = Category
    template_name = 'td_cms/category_detail.html'


class PageDetailView(TranslatedSlugMixin, DetailView):
    model = Page
    template_name = 'td_cms/page_detail.html'



# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test views
"""
from django.core.urlresolvers import reverse
from django.test import TestCase

from ..models import Page
from .mixins import TDCMSTestMixin


class CategoryDetailViewTests(TDCMSTestMixin, TestCase):
    """
    Tests for the CategoryDetailView
    """
    def setUp(self):
        self._set_category()
        self._add_pages()

    def test_get(self):
        """
        Test the CategoryDetailView get method
        """
        response = self.client.get(self.category.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['category'], self.category)

    def test_get_unknown_slug(self):
        """
        Test the CategoryDetailView get method with an unknown object
        """
        kwargs = {
            'slug_en': 'this-is-a-wrong-slug',
        }
        url = reverse('category_detail_en', kwargs=kwargs)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class PageDetailViewTests(TDCMSTestMixin, TestCase):
    """
    Tests for the CategoryDetailView
    """
    def setUp(self):
        self._set_category()
        self._add_pages()

    def test_get(self):
        """
        Test the PageDetailView get method
        """
        page = Page.objects.get(id=1)
        response = self.client.get(page.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['page'], page)

    def test_get_unknown_slug(self):
        """
        Test the PageDetailView get method with an unknown object
        """
        kwargs = {
            'category_slug': self.category.slug,
            'slug_en': 'this-is-a-wrong-slug',
        }
        url = reverse('page_detail_en', kwargs=kwargs)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)



# -*- coding: utf-8 -*-
import logging

from datetime import datetime
from django import template
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext as _

from modeltranslation.settings import AVAILABLE_LANGUAGES

from td_cms.models import Category, Page

register = template.Library()
logger = logging.getLogger(__name__)


@register.assignment_tag
def get_parent_categories():
    """
    Add TD CMS page categories to the context
    """
    return Category.visible_objects.all()


@register.assignment_tag
def get_highlighted_pages():
    """
    Add TD CMS highlighted pages to the context
    """
    return Page.highlighted_objects.all()


@register.assignment_tag
def get_category(slug):
    """
    Add a category to the context given its slug
    """
    for language_code in AVAILABLE_LANGUAGES:
        slug_field = 'slug_%s' % language_code
        try:
            return Category.objects.get(**{slug_field: slug})
        except:
            pass
    logger.warning("Cannot resolve category slug '%s'" % slug)
    return None


@register.assignment_tag
def get_page(slug):
    """
    Add a page to the context given its slug
    """
    for language_code in AVAILABLE_LANGUAGES:
        slug_field = 'slug_%s' % language_code
        try:
            return Page.objects.get(**{slug_field: slug})
        except:
            pass
    logger.warning("Cannot resolve page slug '%s'" % slug)
    return None


@register.filter
def timestamp(value):
    """
    Convert a datetime object representation to a unix timestamp.

    If the object to filter is not a valid datetime.datetime instance, returns
    the original unmodified object.
    """
    if not isinstance(value, datetime):
        return value
    return value.strftime("%s")


@register.inclusion_tag('td_cms/partials/comments/disqus_thread.html')
def show_disqus_thread(page):
    """
    Show the disqus thread for a Page that allows comments
    """
    from td_cms import settings
    # We force settings reloading for testing purpose: when we change the
    # TD_CMS_DISQUS_SHORTNAME in django settings, we need to update td_cms
    # settings
    reload(settings)

    if not settings.DISQUS_SHORTNAME:
        msg = _('TD_CMS_DISQUS_SHORTNAME parameter is missing in your '
                'settings')
        raise ImproperlyConfigured(msg)

    return {
        'page': page,
        'disqus_shortname': settings.DISQUS_SHORTNAME,
    }



# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test models.
"""
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.test import TestCase
from django.utils.text import slugify
from django.utils.translation import activate

from ..factories import PageFactory
from ..models import Category, Page
from .mixins import TDCMSTestMixin


class CategoryModelTests(TDCMSTestMixin, TestCase):
    """
    Tests for the Category model
    """
    def setUp(self):
        self._set_category()

    def test_unicode(self):
        """
        Test __unicode__ method
        """
        self.assertEqual(unicode(self.category), self.category.name)

    def test_saving_and_retrieving_items(self):
        """
        Test saving and retrieving two categories with different names
        """
        name_1 = self.category.name

        name_2 = u"Category 2"
        category_2 = Category()
        category_2.name = name_2
        category_2.save()

        categories = Category.objects.all()
        self.assertEqual(categories.count(), 2)

        saved_category_1 = categories[0]
        saved_category_2 = categories[1]
        self.assertEqual(saved_category_1.name, name_1)
        self.assertEqual(saved_category_2.name, name_2)

    def test_slug_unicity(self):
        """
        Test saving two categories with the same name
        """
        category_2 = Category()
        category_2.name = self.category.name
        self.assertRaisesMessage(
            IntegrityError,
            'column slug_en is not unique',
            category_2.save
        )

    def test_multilingual_slug(self):
        """
        Test multilingual slugification
        """
        name_en = u"Django books"
        name_fr = u"Livres de Django"

        category = Category()
        category.name_en = name_en
        category.name_fr = name_fr

        category.slugify('name')
        self.assertEqual(category.slug_en, slugify(name_en))
        self.assertEqual(category.slug_fr, slugify(name_fr))

    def test_get_absolute_url(self):
        """
        Test the get_absolute_url method (with multilingual support)
        """
        url = reverse('category_detail_en', args=[self.category.slug])
        self.assertEqual(self.category.get_absolute_url(), url)

        # Set a new language
        activate('fr')
        url = reverse('category_detail_fr', args=[self.category.slug])
        self.assertEqual(self.category.get_absolute_url(), url)

    def test_category_mptt_order(self):
        """
        Test Category tree ordering
        """
        self.category.position = 1
        self.category.save()

        name_1 = self.category.name

        name_2 = u"Must be the first"
        category_2 = Category()
        category_2.name = name_2
        category_2.position = 0
        category_2.save()

        categories = Category.objects.all()

        first_position_category = categories[0]
        second_position_category = categories[1]
        self.assertEqual(first_position_category.name, name_2)
        self.assertEqual(second_position_category.name, name_1)

    def test_get_pages(self):
        """
        Test the get_pages method
        """
        self._add_pages()

        pages = self.category.get_pages()
        self.assertEqual(len(pages), 3)

    def test_get_pages_with_subcategories(self):
        """
        Test the get_pages method when the main category has no linked pages
        but one of its sub-categories has linked pages.
        """
        # Create two sub-categories with 6 pages (3 highlighted pages)
        for num in range(1, 3):
            subcategory = Category.objects.create(
                name=u"Category 1.%(num)d" % {'num': num},
                parent=self.category
            )
            self._add_pages(subcategory)

        pages = self.category.get_pages()
        self.assertEqual(len(pages), 6)

    def test_get_highlighted_pages(self):
        """
        Test the get_highlighted_pages method
        """
        self._add_pages()
        page = self.category.page.all()[0]
        page.is_highlighted = True
        page.status = 'published'
        page.save()

        pages = self.category.get_highlighted_pages()
        self.assertEqual(len(pages), 1)

    def test_visible_category_manager(self):
        """
        Test the visible category manager
        """
        self._set_category_tree()
        categories = Category.objects.all()
        visible_categories = Category.visible_objects.all()

        # By default a category is visible
        self.assertEqual(categories.count(), visible_categories.count())

        # Toggle all categories visibility status
        Category.objects.all().update(visible_in_menu=False)
        visible_categories = Category.visible_objects.all()
        self.assertEqual(visible_categories.count(), 0)


class PageModelTests(TDCMSTestMixin, TestCase):
    """
    Tests for the Category model
    """
    def setUp(self):
        self._set_category()
        self._set_page(categories=[self.category])

    def test_unicode(self):
        """
        Test __unicode__ method
        """
        self.assertEqual(unicode(self.page), self.page.title)

    def test_page_factory(self):
        """
        Test the PageFactory with no linked categories
        """
        page = PageFactory.build(title=u"This is a title")
        self.assertIsNot(page, None)

    def test_saving_and_retrieving_items(self):
        """
        Test saving and retrieving two categories with different names
        """
        title_1 = self.page.title

        title_2 = u"Title 2"
        page_2 = Page()
        page_2.title = title_2
        page_2.save()

        pages = Page.objects.all()
        self.assertEqual(pages.count(), 2)

        saved_page_1 = pages[0]
        saved_page_2 = pages[1]
        self.assertEqual(saved_page_1.title, title_1)
        self.assertEqual(saved_page_2.title, title_2)

    def test_get_absolute_url(self):
        """
        Test the get_absolute_url method (with multilingual support)
        """
        page_slug = self.page.slug
        category_slug = self.page.categories.all()[0].slug

        url = reverse('page_detail_en', args=[category_slug, page_slug])
        self.assertEqual(self.page.get_absolute_url(), url)

        # Set a new language
        activate('fr')
        url = reverse('page_detail_fr', args=[category_slug, page_slug])
        self.assertEqual(self.page.get_absolute_url(), url)

    def test_get_current_site(self):
        """
        Test the get_current_site method from the SiteMixin
        """
        site = self.page.get_current_site()
        self.assertEqual(site.domain, 'example.com')
        self.assertEqual(site.name, 'example.com')

    def test_get_image_full_url(self):
        """
        Test the get_image_full_url method
        """
        # No image has been defined
        image_full_url = self.page.get_image_full_url()
        self.assertEqual(image_full_url, '')

        # Define an image
        image_relative_url = '/media/upload/foo/test.jpg'
        self.page.image = image_relative_url
        self.page.save()
        image_full_url = self.page.get_image_full_url()

        site = self.page.get_current_site()
        url = "http://%(domain)s%(image)s" % {
            'domain': site.domain,
            'image': image_relative_url,
        }
        self.assertEqual(url, image_full_url)

    def test_published_page_manager(self):
        """
        Test the published page manager
        """
        self._set_category_tree()
        published_pages = Page.published_objects.all()

        self.assertEqual(published_pages.count(), 18)

    def test_highlighted_page_manager(self):
        """
        Test the highlighted page manager
        """
        self._set_category_tree()
        highlighted_pages = Page.highlighted_objects.all()

        self.assertEqual(highlighted_pages.count(), 4)
